#!/usr/bin/perl


##########################################################
# Converts simple formatted ASCII Texts to DocBook XML
#
# (c) Thomas Weber 2002
#
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Please see the file 'LICENSE.txt' for detailed copyright description 
# and the disclaimer  
#
# Version: $Id: txt2docbook.pl,v 1.4 2003/05/19 17:27:47 tweber Exp $
##########################################################

##########################################
# Global vars and imports 
#{{{######################################

use Getopt::Std;
use IO::File;
use blocks;

my @_CURRENTBLOCKS;		# contains a list of the current block list (including the active one)

##########################################
# Returns the currently active block
##########################################
sub getCurrentblock {
	return $_CURRENTBLOCKS[$#_CURRENTBLOCKS];	
}
#}}}

##########################################
# Print help and exit program
# {{{ ####################################
sub printhelp {
	print STDERR "
Usage:
	$0 [options] <filename>

Options:
	-h Print this help and exit
	-o Use a specific output module instead of the default docbook mode.
";
	exit(1);
 
}

# }}}

##########################################
# Handle a new Section
#{{{######################################
sub doSection {
	my ($indents,$depth,$title)=@_;
	my $block;
	my @sections;
	my $done;
	
	# open new sublevel
	$block=Section->new($indents,$depth,$title);
	doBlockend($block);
	$block->begin();
	
	push @_CURRENTBLOCKS,$block;

	
}
#}}}

##########################################
# Handle empty line
#{{{######################################
sub doEmpty {
	
	doBlockend();
	
}
#}}}

##########################################
# Handle block end
# {{{ ####################################
sub doBlockend {

	my $newblock=shift;
	my $lastblock;
	my $success;
	
	do {
		$success=0;
		$lastblock=getCurrentblock();
		
		return if (!$lastblock);
			
		if ( $lastblock->end($newblock) ) {
			pop @_CURRENTBLOCKS;
			$success=1;
		}
		
	} while ($success);
	
	
}

#}}}

##########################################
# Handle a textline
#
# This method deals also with the document
# header, title and author.
# {{{ ####################################

sub doText {
	
	my $indents=shift;
	my $text=shift;
	my $lastblock=getCurrentblock();
	my $block;
	
	# Document has not been started yet, create the document block
	if (! ref($lastblock) ) {
		$block=Document->new();	
		$block->begin();
		push @_CURRENTBLOCKS,$block;
		# The Document contains always a DocumentTitle block. 
		# Dont print it yet because we want the chance to add the authors name which we don't know yet.
		$block=DocumentTitle->new($text);
		push @_CURRENTBLOCKS,$block;
		return;
	}
	
	# If we are inside the DocumentTitle block, add the current (2. line of the document) as author.
	if (ref($lastblock) eq "DocumentTitle") {
		$lastblock->{author}=$text;
		return;
	}	
	
	# Begin a new paragraph if we are not already in one section that may contain additional lines of text.
	if (ref($lastblock) ne "Paragraph" && ref($lastblock) ne "ListItem" && ref($lastblock) ne "VarListItem" ) { 
		# begin a new paragraph
		$block=Paragraph->new($indents);
		doBlockend($block);
		$block->begin($text);
		push @_CURRENTBLOCKS,$block;
		
	} else {
		# Output the text
		output::textOutput($text);
	}
	
	
}

# }}}

##########################################
# Handle item line
# {{{ ####################################

sub doItem {
	
	my $indents=shift;
	my $text=shift;
	my $block;
	my $doitem;
	my $dolist;
	my $sublist;
	
	$block=ListItem->new($indents);
	
	doBlockend($block);
	
	# check if the parent is a list
	my $lastblock=getCurrentblock();
	
	if (ref($lastblock) eq "List") {
		# already on a list? 
		
		# check indentation and create new sublist as needed
		if ($indents > $lastblock->{indent}) {
			$dolist=1;
			$sublist=1;
			
		}
		
		$doitem=1;
	
	} else {
		# create a new list
		$dolist=1;
		$doitem=1;
	}

	if ($dolist) {
		# begin new list
		$block=List->new($indents);
		$block->{sublist}=1 if ($sublist);
		$block->begin();
		push @_CURRENTBLOCKS,$block;
	}
	
	if ($doitem) {	
		# create the item
		$block=ListItem->new($indents);
		$block->begin($text);
		push @_CURRENTBLOCKS,$block;
	}
	
	
}
# }}}

##########################################
# Handle varitem line
# {{{ ####################################

sub doVarItem {
	
	my $indents=shift;
	my $term=shift;
	my $text=shift;
	my $block;
	my $doitem;
	my $dolist;
	my $sublist;
	
	
	$block=ListItem->new($indents);
	
	doBlockend($block);
	
	# check if the parent is a list
	my $lastblock=getCurrentblock();
	
	if (ref($lastblock) eq "VarList") {
		# already on a list? 
		
		# check indentation and create new sublist as needed
		if ($indents > $lastblock->{indent}) {
			$dolist=1;
			$sublist=1;
			
		}
		
		$doitem=1;
	
	} else {
		# create a new list
		$dolist=1;
		$doitem=1;
	}

	if ($dolist) {
		# begin new list
		$block=VarList->new($indents);
		$block->{sublist}=1 if ($sublist);
		$block->begin();
		push @_CURRENTBLOCKS,$block;
	}
	
	if ($doitem) {	
		# create the item
		$block=VarListItem->new($indents);
		$block->begin($term,$text);
		push @_CURRENTBLOCKS,$block;
	}
	
	
}
# }}}


##########################################
# Counts and returns the indentations 
# of the current line
##########################################
sub getindents {
	my $indents=shift;
	
	$indents=~s/\t/_/g;
	$indents=~s/\s//g;
	$indents=length($indents);

	return $indents;	
}
##########################################
##########################################
# MAIN
#{{{######################################

my ($OUTPUTMODULE)="/usr/local/share/perl5/output.pl";

getopts('ho:');

if ( $opt_h ) {
	printhelp;
}

# switch to a specific outputmodule
if ( $opt_o ) {
	$OUTPUTMODULE = "output_$opt_o.pl";
}

# test wether the outputmodule exists
if ( ! -e $OUTPUTMODULE ) {
	print STDERR "Error. The output-module $OUTPUTMODULE does not exist or could not be found within the current path.\n\n";
	printhelp;
}



$filename = shift or printhelp;

my $tagmode;

require "$OUTPUTMODULE";
print STDERR "Reading file $filename. Output module is $OUTPUTMODULE\n";

@filestack=();

$in=new IO::File;
$in->open("< $filename") or die "Can't open file $filename (use -h for help)\n";

do {
	while ($line=<$in>) {
	
	# {{{ Tagblocks
	if ($line =~ /^(\s*)\<(\/?\w+)\s*.*\>(.*)/) {
		# arbitrary docbook tag
		$indents=getindents($1);
		$tagname=$2;
		$text=$line;
		$tagname=lc($tagname);
		
		
		if ($tagmode) {
			
			if ('/'.$tagmode eq $tagname ) {
				$tagmode=0;
				output::endPassthrough($tagname,$line);
			} else {
				output::PassthroughOutput($tagmode,$line);	
			}
			
		} else {
			
			$tagmode=lc($tagname);
			output::startPassthrough($tagmode,$line);
		}
		
		next;
	} 
		
	if ($tagmode) {
		output::PassthroughOutput($tagmode,$line);
		next;
	}
	# }}}
	
	chop $line;
	
	if ($line =~ /^\s*&(.*);$/) {
		# include another source file at this position
		$filename=$1;
		if (-e $filename ) {

			# Put current filehandle on the stack
			push @filestack,$in;
			
			$in=new IO::File($filename,'r');
			if ($in) {
				# Successfull opened? continue with reading the include
				next;
			} else {
				# Any Problem? Get the parent file and continue.
				$in=shift @filestack;
			}
		} 

		print STDERR "Error opening the include $filename\n";

	}

	# Convert < and > to the appropriate html entities
    $line=~ s,>,&#62;,gi;
    $line=~ s,<,&#60;,gi;
    $line=~ s,\",&#34;,gi;
    $line=~ s,\',&#39;,gi;
    $line=~ s,\/,&#47;,gi;
    $line=~ s,\[,&#91;,gi;
    $line=~ s,\],&#93;,gi;
    $line=~ s,\\,&#92;,gi;
    $line=~ s,\{,&#123;,gi;
    $line=~ s,\},&#125;,gi;

	if ($line =~ /^\s*$/) {
		# identify empty line
		doEmpty();
	} elsif ($line =~ /^(\s*)([\*|\d+\.)]+)\s+(.*)/) {
		# identify section		
		$indents=getindents($1);
		
		$depth=$2;
		$title=$3;
		
		if ($depth =~ /\*+/) {
			# '*' is used to mark sections
			$depth=length($depth);
		} else {
			# '1.1.1' is used to mark sections
			@list=split /\./,$depth;
			print STDERR $depth."\n";
			print STDERR "=".($#list+1)."\n";
			$depth=$#list+1;
		}
		doSection($indents,$depth,$title);
	
	} elsif ($line =~ /^(\s*)[-|o|\+|=]\s+(.*)/) {
		# identify list item
		$indents=getindents($1);
		$text=$2;
		doItem($indents,$text);
		
	} elsif ($line =~ /^(\s*)\[(.*)\](\s+(.*))?/) {
		# identify varlist
		$indents=getindents($1);
		$term=$2;
		$text=$4;
		doVarItem($indents,$term,$text);
		
	}  elsif ($line =~ /^(\s*)(.*)/) {
		# this is text
		$indents=getindents($1);
		$text=$2;
		doText($indents,$text);
	}
	
	}
	$in->close;
	$in=shift @filestack;
	# Check if this was only a included child document

} while ($in);

doBlockend("ENDDOCUMENT");
print STDERR "Done!\n";

#}}}
# :folding=explicit:
# :collapseFolds=1:	
